﻿using System;

using Butterfly.Messages;
using Butterfly.HabboHotel.Rooms;
using Butterfly;

namespace Butterfly.HabboHotel.Navigators
{
    internal enum PublicImageType
    {
        INTERNAL = 0,
        EXTERNAL = 1
    }

    internal enum PublicItemType
    {
        //TAG(1), FLAT(2), PUBLIC_FLAT(3), CATEGORY(4);
        NONE = 0,
        TAG = 1,
        FLAT = 2,
        PUBLIC_FLAT = 3,
        CATEGORY = 4
    }

    internal class PublicItem
    {
        private readonly Int32 BannerId;

        internal int Type;

        internal string Caption;
        internal string Image;
        internal PublicImageType ImageType;

        internal UInt32 RoomId;

        internal Int32 ParentId;

        internal Int32 Id
        {
            get { return BannerId; }
        }

        internal RoomData RoomData
        {
            get
            {
                if (RoomId == 0)
                {
                    return new RoomData();
                }

                if (ButterflyEnvironment.GetGame() == null)
                    throw new NullReferenceException();

                if (ButterflyEnvironment.GetGame().GetRoomManager() == null)
                    throw new NullReferenceException();

                if (ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId) == null)
                    throw new NullReferenceException();

                return ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId);
            }
        }

        internal string Description;
        internal Boolean Recommended;
        internal int CategoryId;
        internal PublicItemType itemType;
        internal string TagsToSearch;

        internal PublicItem(int mId, int mType, string mCaption, string mDescription, string mImage, PublicImageType mImageType, uint mRoomId, int mCategoryId, int mParentId, Boolean mRecommand, int mTypeOfData, string mTags)
        {
            BannerId = mId;
            Type = mType;
            Caption = mCaption;
            Description = mDescription;
            Image = mImage;
            ImageType = mImageType;
            RoomId = mRoomId;
            ParentId = mParentId;
            CategoryId = mCategoryId;
            Recommended = mRecommand;
            if (mTypeOfData == 1)
                itemType = PublicItemType.TAG;
            else if (mTypeOfData == 2)
                itemType = PublicItemType.FLAT;
            else if (mTypeOfData == 3)
                itemType = PublicItemType.PUBLIC_FLAT;
            else if (mTypeOfData == 4)
                itemType = PublicItemType.CATEGORY;
            else
                itemType = PublicItemType.NONE;
        }

        internal RoomData RoomInfo
        {
            get
            {
                try
                {
                    if (RoomId > 0)
                        return ButterflyEnvironment.GetGame().GetRoomManager().GenerateRoomData(RoomId);
                    else
                        return null;
                }
                catch
                {
                    return null;
                }
            }
        }

        internal void Serialize(ServerMessage Message)
        {
            try
            {
                Message.AppendInt32(Id);
                Message.AppendString(this.itemType != PublicItemType.PUBLIC_FLAT ? Caption : String.Empty);
                Message.AppendString(Description); // description
                Message.AppendInt32(Type); // image size
                Message.AppendString(this.itemType == PublicItemType.PUBLIC_FLAT ? Caption : String.Empty);
                Message.AppendString(Image);
                Message.AppendInt32(ParentId > 0 ? ParentId : 0);
                Message.AppendInt32(RoomInfo != null ? RoomInfo.UsersNow : 0);
                Message.AppendInt32(itemType == PublicItemType.NONE ? 0 : (itemType == PublicItemType.TAG ? 1 : (itemType == PublicItemType.FLAT ? 2 : (itemType == PublicItemType.PUBLIC_FLAT ? 2 : (itemType == PublicItemType.CATEGORY ? 4 : 0)))));

                if (this.itemType == PublicItemType.TAG)
                {
                    Message.AppendString(TagsToSearch); return;
                }
                else if (this.itemType == PublicItemType.CATEGORY)
                {
                    Message.AppendBoolean(false); return;
                }
                else if (this.itemType == PublicItemType.FLAT)
                {
                    this.RoomInfo.Serialize(Message, false);
                }
                else if (this.itemType == PublicItemType.PUBLIC_FLAT)
                {
                    this.RoomInfo.Serialize(Message, false);
                }
                /*if (!Category)
                {
                    Message.AppendInt32(Id);

                    Message.AppendStringWithBreak((Type == 1) ? Caption : RoomData.Name);

                    Message.AppendStringWithBreak(RoomData.Description);
                    Message.AppendInt32(Type);
                    Message.AppendStringWithBreak(Caption);
                    Message.AppendStringWithBreak((ImageType == PublicImageType.EXTERNAL) ? Image : string.Empty);
                    Message.AppendInt32(ParentId);
                    Message.AppendInt32(RoomData.UsersNow);
                    Message.AppendInt32(3);
                    Message.AppendStringWithBreak((ImageType == PublicImageType.INTERNAL) ? Image : string.Empty);
                    Message.AppendUInt(1337);
                    Message.AppendBoolean(true);
                    Message.AppendStringWithBreak(RoomData.CCTs);
                    Message.AppendInt32(RoomData.UsersMax);
                    Message.AppendUInt(RoomId);
                }
                else if (Category)
                {
                    Message.AppendInt32(Id);
                    Message.AppendStringWithBreak(Caption);
                    Message.AppendStringWithBreak(string.Empty);
                    Message.AppendBoolean(true);
                    Message.AppendStringWithBreak(string.Empty);
                    Message.AppendStringWithBreak(Image);
                    Message.AppendBoolean(false);
                    Message.AppendBoolean(false);
                    Message.AppendInt32(4);
                    Message.AppendBoolean(false);
                }  */
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception on publicitems composing: " + e.ToString());
            }
        }
    }
}