﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SharedPacketLib.DataPackets;
using SharedPacketLib.Data_Format;
using System.Collections;

namespace ClientImplementation.ConnectionClasses
{
    public class PacketProcessor 
    {
        private PacketRegistery<ClientOpCode, ClientIncomingPacket> immidiateHandeler;
        private PacketRegistery<ClientOpCode, ClientIncomingPacket> queueHandeler;
        private Queue packetQueue;
        
        /// <summary>
        /// Creates a new packet processor
        /// </summary>
        public PacketProcessor()
        {
            this.immidiateHandeler = new PacketRegistery<ClientOpCode, ClientIncomingPacket>();
            this.queueHandeler = new PacketRegistery<ClientOpCode, ClientIncomingPacket>();
            packetQueue = new Queue();
        }


        /// <summary>
        /// Removes a packet from the packet handeler
        /// </summary>
        /// <param name="code">The opcode that is removed from the game</param>
        public void removePacket(ClientOpCode code)
        {
            immidiateHandeler.removePacket(code);
            queueHandeler.removePacket(code);
        }

        /// <summary>
        /// registers a packet
        /// </summary>
        /// <param name="opCode">Server opcode</param>
        /// <param name="handeler">Handeler</param>
        public void registerPacket(ClientOpCode opCode, PacketRegistery<ClientOpCode, ClientIncomingPacket>.handlePacket handeler, bool handleImmidiate = false)
        {
            if (handleImmidiate)
            {
                immidiateHandeler.registerPacket(opCode, handeler);
            }
            else {
                this.queueHandeler.registerPacket(opCode, handeler);
            }
        }

        /// <summary>
        /// Handles a packet using the correct queue
        /// </summary>
        /// <param name="packet">The packet that needs to be processed</param>
        public void handlePacket(ClientIncomingPacket packet)
        {
            PacketRegistery<ClientOpCode, ClientIncomingPacket>.handlePacket handeler = immidiateHandeler.getHandeler(packet.Opcode);
            if (handeler != null)
                handeler.Invoke(packet);
            else
            {
                lock (packetQueue.SyncRoot)
                {
                    packetQueue.Enqueue(packet);
                }
            }
            
        }

        /// <summary>
        /// Handles a packet
        /// </summary>
        public void processPackets()
        {
            if (packetQueue.Count != 0)
            {
                lock (packetQueue.SyncRoot)
                {
                    ClientIncomingPacket p;
                    PacketRegistery<ClientOpCode, ClientIncomingPacket>.handlePacket handeler;
                    while (packetQueue.Count != 0)
                    {
                        p = (ClientIncomingPacket)packetQueue.Dequeue();
                        handeler = this.queueHandeler.getHandeler(p.Opcode);
                        if (handeler != null) // If there is a handeler for it..
                            handeler.Invoke(p);
                    }
                }
            }
        }


    }
}
