﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ConnectionManager;
using SharedPacketLib.DataPackets;
using SharedPacketLib.Data_Format;
using SocketTest.Packets;
using SocketTest.Managers;

namespace SocketTest
{
    public class ConnectionHandeler
    {
        private ConnectionManager.ConnectionInformation connection;
        private bool isConnected = true;
        private bool disconnecting = false;
        private PacketRegistery<ServerOpCode, ServerIncomingPacket> packetRegister;

        public delegate void OnDisconnect(ConnectionHandeler handeler);
        public event OnDisconnect Disconnection;
        private LicenseHolder licenseCarry;
        private PingChecker pingCheck;

        public ConnectionHandeler(ConnectionManager.ConnectionInformation connection)
        {
            this.connection = connection;
            this.packetRegister = new PacketRegistery<ServerOpCode, ServerIncomingPacket>();
            packetRegister.registerPacket(ServerOpCode.Core_client_sends_version, ServerReceivedVersion);
            this.connection.sendData(new SharedPacketLib.DataPackets.ServerOutgoingPacket(ClientOpCode.Core_send_version));
            this.connection.connectionChanged += connection_connectionChanged;
            (this.connection.parser as GamePacketParser).packetArrival += ConnectionHandeler_packetArrival;
            this.connection.startPacketProcessing();
            this.pingCheck = new PingChecker(this);
        }

        public void addPacketHandeler(ServerOpCode opCode, PacketRegistery<ServerOpCode, ServerIncomingPacket>.handlePacket handlePacket)
        {
            this.packetRegister.registerPacket(opCode, handlePacket);
        }

        public void removeHandeler(ServerOpCode opCode)
        {
            this.packetRegister.removePacket(opCode);
        }

        private void ConnectionHandeler_packetArrival(ServerIncomingPacket packet)
        {
            PacketRegistery<ServerOpCode, ServerIncomingPacket>.handlePacket handeler = packetRegister.getHandeler(packet.Opcode);
            if (handeler != null)
                handeler.Invoke(packet);
        }

        private void connection_connectionChanged(ConnectionInformation information, ConnectionState state)
        {
            switch (state)
            {
                case ConnectionState.closed:
                    disconnect();
                    break;
            }
        }

        private void disconnect()
        {
            if (this.isConnected && !this.disconnecting)
            {
                this.Dispose();
                disconnecting = true;
                this.isConnected = false;
                this.pingCheck = null;
                this.connection.connectionChanged -= connection_connectionChanged;
                this.connection.Dispose();
                if (this.Disconnection != null)
                    this.Disconnection.Invoke(this);
            }
        }

        private void ServerReceivedVersion(ServerIncomingPacket packet)
        {
            int version = packet.ReadInt();
            int realVersion = AwsomeLicense.Configuration.getInt("version.id");
            if (version == realVersion)
            {
                sendPacket(new VersionAproved());
                this.licenseCarry = new LicenseHolder(this);
            }
            else
            {
                sendPacket(new VersionOutOfDatePacket());
                this.Dispose();
            }
        }

        internal string getIp()
        {
            return this.connection.getIp();
        }

        internal void Start()
        {
            this.connection.startPacketProcessing();
        }

        internal void sendPacket(ISendable sendItem)
        {
            this.connection.sendData(sendItem.getPacket() as ServerOutgoingPacket);
        }

        internal void Dispose()
        {
            if (this.licenseCarry != null)
                this.licenseCarry.Dispose();
            if (this.pingCheck != null)
                this.pingCheck.Dispose();
            this.connection.Dispose();
        }

        public int ID { get { return this.connection.getConnectionID(); } }

        internal ConnectionInformation getConnection()
        {
            return this.connection;
        }

        internal long getPing()
        {
            if (this.pingCheck != null)
            {
                return pingCheck.getPing();
            }
            else return 9999;
        }
    }
}
