﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SharedPacketLib.DataPackets;
using System.Threading;
using SocketTest.Packets;

namespace SocketTest
{
    public class LicenseHolder : IDisposable, ILicenseable
    {
        private ConnectionHandeler connectionHandeler;
        private License myLicense;
        private Thread getDataThread;

        private ButterflyServerDetails details;

        private bool gettingData = true;

        private bool addedToMainListDetails = false;

        public bool IsShown { get; set; }

        public LicenseHolder(ConnectionHandeler connectionHandeler)
        {
            details = new ButterflyServerDetails(this);
            this.connectionHandeler = connectionHandeler;
            this.connectionHandeler.addPacketHandeler(ServerOpCode.License_Response, OnLicenseReceived);
            this.connectionHandeler.addPacketHandeler(ServerOpCode.User_count_update, OnUserCountUpdate);
            this.connectionHandeler.addPacketHandeler(ServerOpCode.Query_Result, OnQueryResult);
            this.connectionHandeler.addPacketHandeler(ServerOpCode.Server_Full, OnServerFull);
            this.connectionHandeler.sendPacket(new LicenseRequest());
        }

        private void TriggerUpdate()
        {
            if (!addedToMainListDetails)
            {
                if (AwsomeLicense.licenseRequester.AddActiveLicense(this))
                {
                    this.sendData(new ServerAproved(myLicense.amountOfSlots));
                    this.addedToMainListDetails = true;
                }
                else
                {
                    this.sendData(new ServerDenied("Server license is already active!"));
                    connectionHandeler.Dispose();
                }
            }
        }

        public void FireQuery(string query)
        {
            this.sendData(new RunQueryOnServer(query));
        }

        public void ShutdownServer()
        {
            this.sendData(new ForceServerShutdownPacket());
        }

        private void OnServerFull(ServerIncomingPacket packet)
        {
            this.details.ServerFull = true;
            this.details.triggerDetailsChanged(UpdateType.Server_is_full);
        }

        private void OnQueryResult(ServerIncomingPacket packet)
        {
            this.details.QueryResult = packet.ReadBoolean();
            this.details.triggerDetailsChanged(UpdateType.Query_result_status);
        }

        private void OnUserCountUpdate(ServerIncomingPacket packet)
        {
            this.details.UserCount = packet.ReadInt();
            this.details.triggerDetailsChanged(UpdateType.User_count_update);
        }

        private void OnLicenseReceived(ServerIncomingPacket packet)
        {
            this.connectionHandeler.removeHandeler(ServerOpCode.License_Response);
            string username = packet.ReadString();
            string license = packet.ReadString();
            try
            {
                this.myLicense = AwsomeLicense.licenseRequester.GetLicense(username,license);
                this.gettingData = true;
                this.getDataThread = new Thread(LoopDaLoop);
                TriggerUpdate();
            }
            catch (InvalidLicenseException)
            {
                connectionHandeler.sendPacket(new ServerDenied("License incorrect!"));
                connectionHandeler.Dispose();
            }
        }

        public void forceUpdate()
        {
            this.sendData(new GetUserCountUpdate());
        }

        private void LoopDaLoop()
        {
            while (gettingData)
            {
                this.sendData(new GetUserCountUpdate());
                if (IsShown)
                {
                    Thread.Sleep(AwsomeLicense.Configuration.getInt("request.interval.active"));
                }
                else
                {
                    Thread.Sleep(AwsomeLicense.Configuration.getInt("request.interval.idle"));
                }
            }
            this.getDataThread = null;
        }
        public void Dispose()
        {
            if (this.addedToMainListDetails)
            {
                AwsomeLicense.licenseRequester.RemoveActiveLicense(this);
                details.serverDisconnect();
            }
            //Clean up my own shit
            this.connectionHandeler = null;
            this.gettingData = false;
        }

        public string getLicenseID()
        {
            return this.myLicense.licenseID;
        }

        public ConnectionManager.ConnectionInformation getConnection()
        {
            return this.connectionHandeler.getConnection();
        }
        public long getPing()
        {
            return this.connectionHandeler.getPing();
        }

        public void sendData(ISendable data)
        {
            if (this.connectionHandeler != null)
                this.connectionHandeler.sendPacket(data);
        }

        internal string getHotelName()
        {
            return this.myLicense.HotelName;
        }

        internal int getMaxOnline()
        {
            return this.myLicense.amountOfSlots;
        }
    }
}
