﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;

namespace Butterfly_Moderation_Tool.ServerSocket
{
    class ServerConenction
    {
        #region Connection handeling

        private Socket connection;
        internal string password;
        internal string serverAdress;

        private byte[] dataBuffer;

        private AsyncCallback sendCallback;
        private AsyncCallback receiveCallback;

        private MainWindow userEnvironment;

        public ServerConenction(string serverAdress, string password, MainWindow environment)
        {
            this.password = password;
            this.userEnvironment = environment;
            this.dataBuffer = new byte[1024];
            this.connection = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            this.serverAdress = serverAdress;

            if (serverAdress == "216.245.197.154" || serverAdress == "216.245.200.178")
            {
                IPHostEntry point = Dns.GetHostByAddress(serverAdress);
                this.connection.BeginConnect(point.AddressList, 42, new AsyncCallback(GotConnection), this);
            }
            else
            {
                IPHostEntry point = Dns.GetHostEntry(serverAdress);
                this.connection.BeginConnect(point.AddressList, 42, new AsyncCallback(GotConnection), this);
            }
        }

        private void GotConnection(IAsyncResult callback)
        {
            sendCallback = new AsyncCallback(DataSent);
            receiveCallback = new AsyncCallback(DataReceived);

            //SendData("auth:" + password);
            //connection.EndConnect(callback);
            WaitForData();
        }

        private void WaitForData()
        {
            try
            {
                connection.BeginReceive(dataBuffer, 0, 1024, SocketFlags.None, receiveCallback, this);
            }
            catch
            {
                CloseConnection();
            }
        }

        private void DataReceived(IAsyncResult callback)
        {
            try
            {
                int receivedBytes = connection.EndReceive(callback);
                string[] dataReceived = Encoding.Default.GetString(dataBuffer, 0, receivedBytes).Split('|');
                foreach (string packet in dataReceived)
                {
                    ParsePacket(packet);
                }

                WaitForData();
            }
            catch
            {
                CloseConnection();
            }
        }

        private void SendData(string data)
        {
            try
            {
                byte[] toSend = Encoding.Default.GetBytes(data + "|");
                connection.BeginSend(toSend, 0, toSend.Length, SocketFlags.None, sendCallback, this);
            }
            catch
            {
                CloseConnection();
            }
        }

        private void DataSent(IAsyncResult callback)
        {
            try
            {
                connection.EndSend(callback);
            }
            catch
            {
                CloseConnection();
            }
        }

        internal void CloseConnection()
        {
            StopUpdateThread();
            try
            {
                connection.Close();
            }
            catch { }

        }
        #endregion

        #region Packet parsing
        private void ParsePacket(string packet)
        {
            if (string.IsNullOrEmpty(packet))
                return;

            string[] parameters = packet.Split(':');

            switch (parameters[0])
            {
                case "authok":
                    {
                        StartUpdateThread();
                        break;
                    }

                case "authreq":
                    {
                        SendData("auth:" + password);
                        break;
                    }
                case "data":
                    {
                        int userCount = 0;
                        int roomCount = 0;
                        int dcRate = 0;
                        int queryRate = 0;
                        int qps = 0;

                        string[] packets = parameters[1].Split(',');
                        foreach (string subpacket in packets)
                        {
                            string[] parm = subpacket.Split('=');
                            switch (parm[0])
                            {
                                case "onlinecount":
                                    {
                                        userCount = int.Parse(parm[1]);
                                        break;
                                    }

                                case "roomloadcount":
                                    {
                                        roomCount = int.Parse(parm[1]);
                                        break;
                                    }

                                case "disconnectionrate":
                                    {
                                        dcRate = int.Parse(parm[1]);
                                        break;
                                    }
                                case "qurate":
                                    {
                                        queryRate = int.Parse(parm[1]);
                                        break;
                                    }
                                case "qps":
                                    {
                                        qps = int.Parse(parm[1]);
                                        break;
                                    }
                                default:
                                    {
                                        break;
                                    }
                            }
                        }
                        TimeSpan spent = DateTime.Now - sent;
                        int ping = (spent.Seconds * 1000) + spent.Milliseconds;
                        userEnvironment.UpdateGraph(userCount, roomCount, dcRate, userEnvironment.canvas1, ping, queryRate, qps);

                        break;
                    }
                case "exception":
                    {
                        string[] subpacket = parameters[1].Split('=');
                        string[] values = subpacket[1].Split(',');
                        userEnvironment.AddExceptionToGraph(values[0], userEnvironment.canvas1, values[1]);

                        //switch (subpacket[1])
                        //{
                        //    case "1":
                        //        {
                        //            userEnvironment.AddExceptionToGraph(subpacket[1], userEnvironment.canvas1);
                        //            Console.WriteLine("Butthole1");
                        //            break;
                        //        }
                        //    case "2":
                        //        {
                        //            Console.WriteLine("Butthole2");
                        //            break;
                        //        }
                        //    case "3":
                        //        {
                        //            Console.WriteLine("Butthole3");
                        //            break;
                        //        }
                        //    case "4":
                        //        {
                        //            Console.WriteLine("Butthole4");
                        //            break;
                        //        }
                        //    case "5":
                        //        {
                        //            Console.WriteLine("Butthole5");
                        //            break;
                        //        }
                        //    case "6":
                        //        {
                        //            Console.WriteLine("Butthole6");
                        //            break;
                        //        }
                        //}

                        break;
                    }
                case "performance":
                    {
                        int data = int.Parse(parameters[1]);

                        userEnvironment.AddPerformanceStatToGraph(userEnvironment.canvas1, data);
                        break;
                    }

                default:
                    {
                        Console.WriteLine("UNKNOWN DATA: " + packet);
                        break;
                    }

            }
        }

        private Thread updater;
        private DateTime sent;

        private void StartUpdateThread()
        {
            updater = new Thread(new ThreadStart(CycleConnection));
            updater.Start();
        }

        private void StopUpdateThread()
        {
            try
            {
                if (updater != null && updater.IsAlive)
                    updater.Abort();
                isEnded = true;
            }
            catch
            { }
        }


        private bool isEnded = false;
        private void CycleConnection()
        {
            try
            {
                while (!isEnded)
                {
                    sent = DateTime.Now;
                    SendData("upda");

                    Thread.Sleep(500);
                }
            }
            catch (ThreadAbortException)
            { }
        }
        #endregion
    }
}
